import gleam/dict.{type Dict}
import gleam/dynamic/decode.{type Decoder}
import gleam/erlang/process.{type Subject}
import gleam/http/request
import gleam/option
import gleam/otp/actor
import stratus
import utils/auth.{type AgentToken, AgentAuth}
import utils/request as request_utils
import utils/response as response_utils
import utils/types.{type ApiResponse}

pub type GetSupplyChainResponse {
  GetSupplyChainResponse(export_to_import_map: Dict(String, String))
}

fn get_supply_chain_response_decoder() -> Decoder(GetSupplyChainResponse) {
  use export_to_import_map <- decode.field(
    "exportToImportMap",
    decode.dict(decode.string, decode.string),
  )
  decode.success(GetSupplyChainResponse(export_to_import_map:))
}

pub fn get_supply_chain(
  token: AgentToken,
) -> ApiResponse(GetSupplyChainResponse) {
  let request = request_utils.get(AgentAuth(token), "/market/supply-chain")
  use response <- request_utils.try_send(request)
  case response.status {
    200 ->
      response_utils.data_parser(
        response.body,
        get_supply_chain_response_decoder(),
      )
    _ -> response_utils.error_parser(response.body)
  }
}

pub fn get_departure_events(
  token: AgentToken,
  initial_state: state,
  on_message: fn(state, stratus.Message(user)) ->
    Result(state, process.ExitReason),
  on_close: fn(state) -> Nil,
) -> Result(Subject(stratus.InternalMessage(user)), actor.StartError) {
  stratus.websocket(
    request: request_utils.get(AgentAuth(token), "/my/socket.io")
      |> request.set_body(""),
    init: fn() { #(initial_state, option.None) },
    loop: fn(msg, state, conn) {
      case on_message(state, msg) {
        Ok(state) -> actor.continue(state)
        Error(reason) -> {
          let _ = stratus.close(conn)
          actor.Stop(reason)
        }
      }
    },
  )
  |> stratus.on_close(on_close)
  |> stratus.initialize
}
