import gleam/dynamic/decode.{type Decoder}
import gleam/json.{type Json}
import models/survey_deposit.{type SurveyDeposit}
import models/survey_signature.{type SurveySignature}
import models/survey_size.{type SurveySize}
import models/waypoint_symbol.{type WaypointSymbol}

pub type Survey {
  Survey(
    signature: SurveySignature,
    symbol: WaypointSymbol,
    deposits: List(SurveyDeposit),
    expiration: String,
    size: SurveySize,
  )
}

pub fn encode(data: Survey) -> Json {
  json.object([
    #("signature", json.string(data.signature)),
    #("symbol", json.string(data.symbol)),
    #("deposits", json.array(data.deposits, survey_deposit.encode)),
    #("expiration", json.string(data.expiration)),
    #("size", survey_size.encode(data.size)),
  ])
}

pub fn decoder() -> Decoder(Survey) {
  use signature <- decode.field("signature", survey_signature.decoder())
  use symbol <- decode.field("symbol", waypoint_symbol.decoder())
  use deposits <- decode.field(
    "deposits",
    decode.list(survey_deposit.decoder()),
  )
  use expiration <- decode.field("expiration", decode.string)
  use size <- decode.field("size", survey_size.decoder())
  decode.success(Survey(signature:, symbol:, deposits:, expiration:, size:))
}
