import gleam/dynamic/decode.{type Decoder}
import gleam/option.{type Option}
import models/waypoint_orbital.{type WaypointOrbital}
import models/waypoint_symbol.{type WaypointSymbol}
import models/waypoint_type.{type WaypointType}
import utils/decode as decode_utils

pub type SystemWaypoint {
  SystemWaypoint(
    symbol: WaypointSymbol,
    type_: WaypointType,
    x: Int,
    y: Int,
    orbitals: List(WaypointOrbital),
    orbits: Option(String),
  )
}

pub fn decoder() -> Decoder(SystemWaypoint) {
  use symbol <- decode.field("symbol", waypoint_symbol.decoder())
  use type_ <- decode.field("type", waypoint_type.decoder())
  use x <- decode.field("x", decode.int)
  use y <- decode.field("y", decode.int)
  use orbitals <- decode.field(
    "orbitals",
    decode.list(waypoint_orbital.decoder()),
  )
  use orbits <- decode_utils.field_key_value_optional("orbits", decode.string)
  decode.success(SystemWaypoint(symbol:, type_:, x:, y:, orbitals:, orbits:))
}
