import gleam/dynamic/decode.{type Decoder}
import gleam/option.{type Option}
import models/meta
import utils/types.{type ErrorResponse, type PagedData, ErrorResponse, PagedData}

pub fn data_decoder(decoder: Decoder(data)) -> Decoder(data) {
  use data <- decode.field("data", decoder)
  decode.success(data)
}

pub fn paged_data_decoder(decoder: Decoder(data)) -> Decoder(PagedData(data)) {
  use data <- decode.field("data", decoder)
  use meta <- decode.field("meta", meta.decoder())
  decode.success(PagedData(data:, meta:))
}

pub fn error_response_decoder() -> Decoder(ErrorResponse) {
  use code <- decode.field("code", decode.int)
  use message <- decode.field("message", decode.string)
  use data <- field_key_value_optional("data", decode.dynamic)
  use request_id <- field_key_value_optional("requestId", decode.string)
  decode.success(ErrorResponse(code:, message:, data:, request_id:))
}

pub fn error_decoder() -> Decoder(ErrorResponse) {
  use data <- decode.field("error", error_response_decoder())
  decode.success(data)
}

pub fn enum_decoder(
  parse_enum: fn(String) -> Result(enum, enum),
  expected: String,
) -> Decoder(enum) {
  use enum_str <- decode.then(decode.string)
  case parse_enum(enum_str) {
    Ok(enum) -> decode.success(enum)
    Error(enum) -> decode.failure(enum, expected)
  }
}

pub fn field_value_optional(
  field: String,
  decoder: Decoder(a),
  next: fn(Option(a)) -> Decoder(b),
) {
  decode.field(field, decode.optional(decoder), next)
}

pub fn field_key_optional(
  field: String,
  decoder: Decoder(a),
  next: fn(Option(a)) -> Decoder(b),
) {
  decode.optional_field(
    field,
    option.None,
    decode.map(decoder, option.Some),
    next,
  )
}

pub fn field_key_value_optional(
  field: String,
  decoder: Decoder(a),
  next: fn(Option(a)) -> Decoder(b),
) {
  decode.optional_field(field, option.None, decode.optional(decoder), next)
}
