import gleam/dynamic/decode.{type Decoder}
import gleam/json
import utils/decode as decode_utils
import utils/types.{type ApiResponse}

pub fn parser(body: BitArray, decoder: Decoder(data)) -> ApiResponse(data) {
  case json.parse_bits(body, decoder) {
    Ok(data) -> Ok(data)
    Error(err) -> Error(types.JsonDecodeError(err))
  }
}

pub fn data_parser(body: BitArray, decoder: Decoder(data)) -> ApiResponse(data) {
  parser(body, decode_utils.data_decoder(decoder))
}

pub fn paged_data_parser(
  body: BitArray,
  decoder: Decoder(data),
) -> ApiResponse(types.PagedData(data)) {
  parser(body, decode_utils.paged_data_decoder(decoder))
}

pub fn error_parser(body: BitArray) -> ApiResponse(data) {
  Error(case parser(body, decode_utils.error_decoder()) {
    Ok(err) -> types.ResponseError(err)
    Error(err) -> err
  })
}
